import {fetchAuthenticated} from '@/helpers/retrieveData';
import {useLoginStore} from '@/stores/login';
import type {GroupListResponseModel, ListViewConfig} from '@/helpers/models';


const fetchListViewConfigMocked = (): ListViewConfig => {
  const _config: ListViewConfig = {
    url: 'url',
    allowedGlobalActions: ['add'],
    searchFormOptions: {
      schools: [
        {
          value: 'DEMOSCHOOL',
          label: 'DEMOSCHOOL',
        }, {
          value: 'School1',
          label: 'Schule 1',
        },
      ],
      divisClasstype: [
        {
          value: 'class',
          label: 'Class',
        },
        {
          value: 'offer',
          label: 'Offer',
        },
      ],
      activationStates: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
      isLmsGroup: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
      isIservGroup: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
    },
    columns: [
      {
        label: 'School year',
        attribute: 'idiSchoolyear',
      },
      {
        label: 'School name',
        attribute: 'displayName',
      },
      {
        label: 'Name',
        attribute: 'name',
      },
      {
        label: 'Group type',
        attribute: 'divisClasstype',
      },
      {
        label: 'Source UID',
        attribute: 'groupSourceUid',
      },
      {
        label: 'Creation date',
        attribute: 'createTimestamp',
        formatter: 'date',
      },
      {
        label: 'Date of last change',
        attribute: 'modifyTimestamp',
        formatter: 'date',
      },
      {
        label: 'Record UID',
        attribute: 'recordUid',
      },
      {
        label: 'Activation date',
        attribute: 'activationDate',
        formatter: 'date',
      },
      {
        label: 'Expiry date',
        attribute: 'expiryDate',
        formatter: 'date',
      },
      {
        label: 'Scheduled deletion date',
        attribute: 'scheduledDeletionDate',
        formatter: 'date',
      },
      {
        label: 'LMS enabled',
        attribute: 'isLmsGroup',
        formatter: 'boolean:LMS enabled:LMS disabled',
      },
      {
        label: 'IServ enabled',
        attribute: 'isIservGroup',
        formatter: 'boolean:IServ enabled:IServ disabled',
      },
    ],
  };
  console.debug('mock config', _config);
  return _config;
};

export const fetchListViewConfig = async(): Promise<ListViewConfig> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchListViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchListViewConfigMocked());
      }, 1000);
    });
  }

  const loginStore = useLoginStore();
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/pages/listView/settings/${loginStore.username}`);
  return await answer.json();
};


const fetchGroupsMocked = (): GroupListResponseModel[] => {
  /* eslint-disable camelcase */
  const group: GroupListResponseModel = {
    id: 'group1',
    allowedActions: ['edit', 'delete'],
    schoolGroupType: 'workgroup',
    attributes: {
      idiSchoolyear: {
        value: 'Year 3',
        access: 'read',
      },
      displayName: {
        value: 'Group 1',
        access: 'read',
      },
      name: {
        value: 'group1',
        access: 'read',
      },
      divisClasstype: {
        value: 'Group type A',
        access: 'read',
      },
      groupSourceUid: {
        value: 'sourceID',
        access: 'read',
      },
      createTimestamp: {
        value: '2020-10-5',
        access: 'read',
      },
      modifyTimestamp: {
        value: '2020-10-05',
        access: 'read',
      },
      recordUid: {
        value: 'recordUID',
        access: 'read',
      },
      activationDate: {
        value: '2020-10-05',
        access: 'read',
      },
      expiryDate: {
        value: '2020-10-05',
        access: 'none',
      },
      scheduledDeletionDate: {
        value: '2020-10-05',
        access: 'none',
      },
      isLmsGroup: {
        value: true,
        access: 'none',
      },
      isIservGroup: {
        value: false,
        access: 'none',
      },
    },
  };
  /* eslint-enable camelcase */

  const _groups: GroupListResponseModel[] = [group];

  for (let x = 2; x < 500; x++) {
    const y = JSON.parse(JSON.stringify(group));
    y.id = `group${x}`;
    y.attributes.name.value = y.id;
    if (x % 2 === 0) {
      y.allowedActions = ['edit'];
    }
    y.schoolGroupType = 'school_class';
    _groups.push(y);
  }
  console.log('mock groups: ', _groups);
  return _groups;
};

export const fetchGroups = async(
  school: string,
  schoolYear: string,
  divisClassType: string,
  source: string,
  activationState: string,
  isLmsGroup: string,
  iServEnabled: string,
  quickSearch: string,
): Promise<GroupListResponseModel[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchGroups. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchGroupsMocked());
      }, 1000);
    });
  }

  const urlSearchParams = new URLSearchParams({});
  if (schoolYear !== '') {
    urlSearchParams.append('schoolYear', schoolYear);
  }
  if (divisClassType !== '') {
    urlSearchParams.append('divisClassType', divisClassType);
  }
  if (source !== '') {
    urlSearchParams.append('source', source);
  }
  if (activationState !== '') {
    urlSearchParams.append('activationState', activationState);
  }
  if (isLmsGroup !== '') {
    urlSearchParams.append('isLmsGroup', (isLmsGroup === 'enabled').toString());
  }
  if (iServEnabled !== '') {
    urlSearchParams.append('isIservGroup', (iServEnabled === 'enabled').toString());
  }
  if (quickSearch !== '') {
    urlSearchParams.append('quickSearch', quickSearch);
  }
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/groups/search/${school}?${urlSearchParams.toString()}`);

  return await answer.json();
};


export const deleteGroups = async(groups: GroupListResponseModel[]): Promise<string[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('deleteGroups. VITE_USE_REAL_BACKEND===false in DEV mode. Doing nothing');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve([groups[0]?.id as string]);
      }, 1000);
    });
  }

  const fails: string[] = [];
  const promises: Promise<Response>[] = [];
  for (const group of groups) {
    const promise = fetchAuthenticated(`/ucsschool/bff-groups/v1/groups/${group.schoolGroupType}/${group.id}`, {
      method: 'DELETE',
    });
    promise.then(res => {
      if (!res.ok) {
        fails.push(group.id);
      }
    });
    promises.push(promise);
  }
  await Promise.all(promises);
  return fails;
};
